Imports System
Imports System.Windows.Forms
Imports MediaPortal.GUI.Library
Imports MediaPortal.Dialogs
Imports System.Text.RegularExpressions

Namespace MyRandomVideo

    Public Class RandomVideo
        Inherits GUIWindow
        Implements ISetupForm

        Dim playlists() As Playlist
        Dim pl_len As Int32

        Dim plHash As Hashtable
        Public configFile As String = Application.StartupPath & "\myplaylists.ini"
        Dim musicTypes As String = ".wma;.mp3;.wav;.ogg"
        Dim videoTypes As String = ".mpg;.avi;.wmv;.asf;.mpeg;.dvr-ms;.asf;.mp4"


        Public Sub New()

        End Sub

#Region "ISetupForm Members"

        Public Function PluginName() As String Implements ISetupForm.PluginName
            Return "Random Videos"
        End Function

        Public Function Description() As String Implements ISetupForm.Description
            Return "Play Random Videos From Selected Folders"
        End Function

        Public Function Author() As String Implements ISetupForm.Author
            Return "GHackett"
        End Function

        Public Sub ShowPlugin() Implements ISetupForm.ShowPlugin
            Dim plConfig As New PlaylistConfigMain
            plConfig.ShowDialog()
        End Sub

        Public Function CanEnable() As Boolean Implements ISetupForm.CanEnable
            Return True
        End Function

        Public Function GetWindowId() As Integer Implements ISetupForm.GetWindowId
            Return 66612345
        End Function

        Public Function DefaultEnabled() As Boolean Implements ISetupForm.DefaultEnabled
            Return True
        End Function

        Public Function HasSetup() As Boolean Implements ISetupForm.HasSetup
            Return True
        End Function

        Public Function GetHome(ByRef strButtonText As String, ByRef strButtonImage As String, ByRef strButtonImageFocus As String, ByRef strPictureImage As String) As Boolean Implements ISetupForm.GetHome
            strButtonText = PluginName()
            strButtonImage = String.Empty
            strButtonImageFocus = String.Empty
            strPictureImage = "hover_my videos.png"
            Return True
        End Function


#End Region

        Public Overrides Property GetID() As Integer
            Get
                Return 66612345
            End Get
            Set(ByVal Value As Integer)

            End Set
        End Property

        Public Overrides Function Init() As Boolean
            Return Load(GUIGraphicsContext.Skin + "\MyRandomVideo.xml")
        End Function

        Protected Overrides Sub OnClicked(ByVal controlId As Int32, ByVal control As GUIControl, ByVal actionType As MediaPortal.GUI.Library.Action.ActionType)
            If controlId = 50 Then
                control = CType(control, MediaPortal.GUI.Library.GUIListControl)
                Dim sli As GUIListItem = GUIListControl.GetSelectedListItem(66612345, 50)
                startPlaylist(CType(plHash(sli.Label), Playlist))
            End If
        End Sub

        Private Sub startPlaylist(ByVal pl As Playlist)
            Dim arr() As String = getPlaylistItemsArray(pl)

            MediaPortal.Playlists.PlayListPlayer.Reset()
            MediaPortal.Playlists.PlayListPlayer.CurrentPlaylist = MediaPortal.Playlists.PlayListPlayer.PlayListType.PLAYLIST_VIDEO_TEMP
            Dim mpPl As MediaPortal.Playlists.PlayList = MediaPortal.Playlists.PlayListPlayer.GetPlaylist(MediaPortal.Playlists.PlayListPlayer.PlayListType.PLAYLIST_VIDEO_TEMP)
            mpPl.Clear()

            Dim i As Int32 = 0

            'DIALOG
            Dim dlg As GUIDialogMenu = CType(GUIWindowManager.GetWindow(CType(GUIWindow.Window.WINDOW_DIALOG_MENU, Int32)), GUIDialogMenu)
            dlg.Reset()
            Dim slID As Int32 = -1
            Dim slStr As String = ""
            dlg.SetHeading("Select # of videos to play")
            dlg.ShowQuickNumbers = False
            dlg.Add("All")
            For i = 1 To min(20, arr.Length)
                dlg.Add(i.ToString())
            Next
            dlg.DoModal(GUIWindowManager.ActiveWindow)
            slID = dlg.SelectedId
            slStr = dlg.SelectedLabelText
            'END DIALOG

            If slID <> -1 Then
                Dim loopInt As Int32 = arr.Length
                If slStr <> "All" And IsNumeric(slStr) Then
                    loopInt = min(loopInt, CInt(slStr))
                End If
                For i = 0 To loopInt - 1
                    Dim newItem As New MediaPortal.Playlists.PlayList.PlayListItem
                    newItem.FileName = arr(i)
                    newItem.Type = MediaPortal.Playlists.PlayList.PlayListItem.PlayListItemType.Video
                    mpPl.Add(newItem)
                Next
                'mpPl.Shuffle()
                MediaPortal.Playlists.PlayListPlayer.PlayNext(True)
            End If
        End Sub

        Private Function min(ByVal a As Int32, ByVal b As Int32) As Int32
            If a < b Then
                Return a
            Else
                Return b
            End If
        End Function


        Protected Overrides Sub OnPageLoad()
            ReDim playlists(0)
            pl_len = 0
            plHash = New Hashtable
            If Not System.IO.File.Exists(configFile) Then
                Dim writer As New System.IO.StreamWriter(configFile, False)
                writer.Write("")
                writer.Close()
            End If

            Dim reader As New System.IO.StreamReader(configFile)
            Dim line As String = ""
            Dim tmpIndex As Int32 = 0
            While True
                line = reader.ReadLine
                If line Is Nothing Then
                    Exit While
                End If
                If line.Trim() = "" Then
                    Exit While
                End If
                tmpIndex = addPlaylist(line.Trim())
                plHash.Add(line.Trim(), playlists(tmpIndex))
                line = reader.ReadLine()
                If line.Trim() = "music" Then
                    playlists(tmpIndex).listType = PlaylistType.Music
                Else
                    playlists(tmpIndex).listType = PlaylistType.Video
                End If
                line = reader.ReadLine()
                If line.StartsWith("ri:") Then
                    playlists(tmpIndex).includeExp = line.Substring(3)
                    line = reader.ReadLine()
                    playlists(tmpIndex).excludeExp = line.Substring(3)
                    line = reader.ReadLine()
                End If
                While line.Trim() <> "*"
                    playlists(tmpIndex).folders.addFolder(line.Trim())
                    line = reader.ReadLine()
                End While

            End While
            reader.Close()

            Dim i As Int32 = 0
            For i = 0 To pl_len - 1
                Dim li As New GUIListItem(playlists(i).filename)
                GUIControl.AddListItemControl(66612345, 50, li)
            Next

        End Sub

        Private Function addPlaylist(ByVal fileName As String) As Int32
            Dim newIndex As Int32 = pl_len
            ReDim Preserve playlists(pl_len)
            pl_len += 1
            playlists(newIndex) = New Playlist(fileName)
            Return newIndex
        End Function

        Private Function getPlaylistItemsArray(ByRef pl As Playlist) As String()

            Dim fl As New FileList
            Dim i As Int32 = 0
            Dim flArr() As String = pl.folders.getFoldersArray()
            For i = 0 To flArr.Length - 1
                addFolder(flArr(i), fl, pl.listType, pl.includeExp, pl.excludeExp)
            Next
            Dim arr() As String = fl.getFileArray()
            randomizeArray(arr)
            Return arr
        End Function

        Private Function addFolder(ByVal folderPath As String, ByRef fl As FileList, ByVal plt As PlaylistType, ByVal includeExp As String, ByVal excludeExp As String)
            'add subFolders
            Dim subFolders() As String = System.IO.Directory.GetDirectories(folderPath)
            Dim i As Int32 = 0
            For i = 0 To subFolders.Length - 1
                addFolder(subFolders(i), fl, plt, includeExp, excludeExp)
            Next

            'add Files
            Dim files() As String = System.IO.Directory.GetFiles(folderPath)
            Dim fTypes As String = ""
            If plt = PlaylistType.Music Then
                fTypes = musicTypes
            Else
                fTypes = videoTypes
            End If
            Dim inc As New Regex(includeExp.ToLower())
            Dim exc As New Regex(excludeExp.ToLower())
            For i = 0 To files.Length - 1
                Dim fExt As String = files(i).Substring(files(i).LastIndexOf("."))
                If fTypes.IndexOf(fExt) <> -1 And inc.IsMatch(files(i).ToLower()) Then
                    If excludeExp = "" Then
                        fl.addFile(files(i))
                    ElseIf Not exc.IsMatch(files(i).ToLower()) Then
                        fl.addFile(files(i))
                    End If
                End If
            Next

        End Function

        Private Sub randomizeArray(ByRef arr() As String)
            Dim i As Int32 = 0
            Dim j As Int32 = 0
            Dim tmp As String
            Dim ln As Int32 = arr.Length

            Randomize()
            For i = 0 To ln - 1

                j = Int((ln - i) * Rnd() + i)

                ' Swap the numbers.
                tmp = arr(i)
                arr(i) = arr(j)
                arr(j) = tmp
            Next
        End Sub

    End Class
End Namespace